/*+--------------------------------------------------------------------------+
 *| Copyright 1992 - 2002 Rational Software Corporation. All Rights Reserved.|
 *| This software contains proprietary and confidential information of       |
 *| Rational and its suppliers. Use, disclosure or reproduction is           |
 *| prohibited without the prior express written consent of Rational.        |
 *+--------------------------------------------------------------------------+
 */
/*  strstream.h -- class strstream declarations */

#ifndef _STRSTREAM_H_
#define _STRSTREAM_H_

#ifndef __cplusplus
#    error C++ header included when compiling in C mode
#endif

#include <iostream.h>

class strstreambuf : public streambuf {

public:
    strstreambuf();
    strstreambuf(int n);
    strstreambuf(void* (*a)(long), void (*f)(void*));
    strstreambuf(char* _s, int, char* _strt=0);
    ~strstreambuf();

    void        freeze(int = 1);
    int isfrozen();
    char*       str();
    virtual int doallocate();
    virtual int overflow(int);
    virtual int underflow();
    virtual streambuf* setbuf(char*, int);
    virtual streampos seekoff(streamoff, ios::seek_dir, int);

private:
    void* (*allocf)(long);
    void  (*freef)(void*);
    short ssbflags;
    enum { dynamic = 1, frozen = 2, unlimited = 4 };
    int   next_alloc;
    char* real_end;

    void        init(char*, int, char*);
};

inline int strstreambuf::isfrozen() { return (ssbflags&frozen) != 0; }


class strstreambase : public virtual ios {
public:
    strstreambuf* rdbuf();
protected:
    strstreambase(char*, int, char*);
    strstreambase();
    ~strstreambase();
private:
    strstreambuf buf;
};

inline strstreambuf* strstreambase::rdbuf() { return &this->buf; }


class istrstream : public strstreambase, public istream {
public:
    istrstream(char*);
    istrstream(char*, int);
    ~istrstream();
};


class ostrstream : public strstreambase, public ostream {
public:
    ostrstream(char*, int, int = ios::out);
    ostrstream();
    ~ostrstream();

    char* str();
    int   pcount();
};

inline char*    ostrstream::str() { return strstreambase::rdbuf()->str(); }
inline int      ostrstream::pcount() 
{
    return strstreambase::rdbuf()->out_waiting();
}


class strstream : public strstreambase, public iostream {
public:
    strstream();
    strstream(char*, int _sz, int _m);
    ~strstream();

    char* str();
};

inline char* strstream::str() { return strstreambase::rdbuf()->str(); }

#endif /* _STRSTREAM_H_ */
